#include<iostream.h>
#include<fstream.h>
#include<signal.h>
#include"buchtrans_impl.h"
#include<mico/CosNaming.h>
#include"corbalogger.h"
#include"control_impl.h"
#include"buchtrans_impl.h"
#include<rpglib.h>
int main_debuglevel=0;

#define RUNPATH "./"
Debugvar dvmt[]=
{
   {"main",&main_debuglevel},
   {NULL,NULL}
};
CorbaLog corbalog;

static CORBA::ORB_ptr orb;
static buch::haltung_var transver;
PgConnection *pgdb=NULL;
static const char *tname="buchhaltung";

/*------------------- Control-Interface ----------------------------*/
class BuchControl_impl: public Control_impl
{
public:
    BuchControl_impl::BuchControl_impl(const char *pname,Debugvarmap _dvm,int dvmsz):Control_impl(pname,_dvm,dvmsz) {}
  void exportnames( CosNaming::NamingContext_ptr nc) 
      throw(::CORBA::SystemException);
};

void BuchControl_impl::exportnames( CosNaming::NamingContext_ptr nc) 
      throw(::CORBA::SystemException)
{
  /*---- Verffentliche unsere Interfaces im Nameserver ----*/  
  CosNaming::Name contextname;
  contextname.length (1);
  contextname[0].id = CORBA::string_dup (tname);
  contextname[0].kind = CORBA::string_dup ("");
  CORBA::Object_var mnc_obj;
  CosNaming::NamingContext_var mnc;
    
  corbalog << "Verffentliche unsere Interfaces im Nameserver ... " << endl;
  try
  {
      mnc_obj = nc->bind_new_context(contextname);
      if(CORBA::is_nil(mnc_obj))
      {
	  corbalog.log(Syslog::Err,"Dialupmanager-Kontext ist NIL");
	  return;
      }
      mnc=CosNaming::NamingContext::_narrow(mnc_obj);
  }
  catch(CosNaming::NamingContext::AlreadyBound_catch &nex)
  {
      corbalog << "Hoppla: Schon gebunden !" << endl;
      mnc_obj = nc->resolve(contextname);
      mnc=CosNaming::NamingContext::_narrow(mnc_obj);
  }
  catch(CosNaming::NamingContext::NotFound_catch &nex)
  {
      corbalog.setprio(Syslog::Err);
      corbalog << nex << ": da " << nex.why << endl;
      corbalog.setprio(Syslog::Debug);
      return;
  }
  catch (CosNaming::NamingContext::CannotProceed_catch &exc)
  {
      corbalog.log(Syslog::Err,"CannotProceed exception");
      return;
  }
  catch (CosNaming::NamingContext::InvalidName_catch &exc)
  {
      corbalog.log(Syslog::Err,"InvalidName exception");
      return;
  }

  try
  {
      CosNaming::Name name;
      name.length(1);
      name[0].id = (const char *)"transaktionen";
      name[0].kind = CORBA::string_dup ("");
      mnc->rebind(name,transver);
  }
  catch(CosNaming::NamingContext::NotFound_catch &nex)
  {
      corbalog.setprio(Syslog::Err);
      corbalog << nex << ": da " << nex.why << endl;
      corbalog.setprio(Syslog::Debug);
  }
  catch (CosNaming::NamingContext::CannotProceed_catch &exc)
  {
      corbalog.log(Syslog::Err,"CannotProceed exception");
  }
  catch (CosNaming::NamingContext::InvalidName_catch &exc)
  {
      corbalog.log(Syslog::Err,"InvalidName exception");
  }
}

void sighandler (int sig)
{
  if (!CORBA::is_nil (orb))
  {
     orb->shutdown (TRUE);
  }
}

void usage(char *progname)
{
    cerr << "Aufruf: " << progname
         << "[mico-args ...] "
         << "-db postgresdatabase"
         << endl;
    exit(1);
}

int main(int argc,char **argv)
{
  orb = CORBA::ORB::_nil ();
  signal (SIGINT, sighandler);
  signal (SIGTERM, sighandler);
  char *prog = rindex(argv[0],'/');
  prog = (prog ? prog+1 : argv[0]);
  char conninfo[128];conninfo[0]='\0';
  char *cip=conninfo;
  int cirest=128;
  // POA-Name faelschen
  const char *implname = NULL;
  char **orbargv = new char*[argc+2+1];
  int orbargc=0;

  for(int i=0;i<argc;i++)
  {
    if(strcmp(argv[i],"-POAImplName")==0 && i+1<argc)
    {
	implname = argv[i+1];
    }
    if(strcmp(argv[i],"-db")== 0)
    {
	if(i+1>=argc) usage(argv[0]);
	i++;
	snprintf(cip,cirest,"dbname=%s",argv[i]);
	int l = strlen(cip);
	cirest-=l;
	cip+=l;
	tname=argv[i];
    }
    else // das muss ein Orb-argument sein ...
    {
	orbargv[orbargc++]=argv[i];
    }
  }
  if(implname==NULL)
  {
      orbargv[orbargc++]="-POAImplName";
      orbargv[orbargc++]="Buchhaltung";
  }
  orbargv[orbargc]=NULL;
  // Datenbank oeffnen
  cerr << "Oeffne Postgresdatenbank mit " << conninfo << endl;
  try
  {
      pgdb=new PgConnection(conninfo);
  }
  catch(PgException &ex)
  {
      cerr << "Kann Datenbank nicht oeffnen: " << ex.what() << endl;
      exit(1);
  }
  // Orb und POA initialisieren
  orb = CORBA::ORB_init(orbargc,orbargv,"mico-local-orb");
  CORBA::Object_var poaobj = orb->resolve_initial_references("RootPOA");
  PortableServer::POA_var rootpoa = PortableServer::POA::_narrow(poaobj);
  PortableServer::POAManager_var mgr = rootpoa->the_POAManager();
  

  /*----- Fabriken fr Values einrichten ---------*/
  transaction_factory * transfac = new transaction_factory;
  orb->register_value_factory ("IDL:buch/transaction:1.0", transfac);

  faelligkeit_factory * faelligfac = new faelligkeit_factory;
  orb->register_value_factory("IDL:buch/faelligkeit:1.0", faelligfac);
  /*----- Baue die objekte ------------------------*/
  haltung_impl *transver_i = new haltung_impl;
  transver = transver_i->_this();
  mgr->activate();
  /* ------------ Nameserver ermitteln ------------*/
  CORBA::Object_var nsobj = orb->resolve_initial_references("NameService");
  if(CORBA::is_nil(nsobj))
  {
    cerr << "Hallo: Initial Service NameService not resolved " << endl;
    exit(1);
  }
  CosNaming::NamingContext_var nc =CosNaming::NamingContext::_narrow (nsobj);

  cerr << "----- ffne Corba-log --------" << endl;
  CosEventChannelAdmin::EventChannel_var syslogev;
 
  try
  {
      CosNaming::Name syslogevname;
      CORBA::Object_var syslogevobj=nc->resolve(StringToName(syslogevname,"syslog"));
      syslogev = CosEventChannelAdmin::EventChannel::_narrow(syslogevobj);
      if(CORBA::is_nil (syslogev))
      {
	  // throw
	  cerr << "syslog hat falschen typ " << endl;
	  exit(1);
      }
  }
  catch(...)
  {
      cerr << "Kann syslog nicht finden !! " << endl;
      exit(1);
  }
      
  corbalog.init(orb,syslogev,"buchhaltung",prog);
  BuchControl_impl *controlservant = new BuchControl_impl(prog,dvmt,sizeof(dvmt)/sizeof(struct Debugvar));


  rootpoa->activate_object(controlservant);
  {
      char ctlpath[128];
      snprintf(ctlpath,128,"%s%s.ctlref",RUNPATH,prog);
      ofstream aus(ctlpath);
      assert(aus);
      aus << orb->object_to_string(controlservant->_this());
  }

  controlservant->exportnames(nc);
  orb->run();
}  






